#pragma once
/** @file BasicDevices.h
	Class definitions for Parameter-based devices

	Additional classes defined in Filter.h
*/
#include "Device.h"
#include "PluginHeader.h"

/** Base class for Parameter-based devices.
	@ingroup devices

	Parameter devices are used to manipulate values (typically scalar or integer, but can be otherwise)
	to be fed to other devices in the network. These are particularily invaluable for macros.
*/
class ParamDevice : public Device {
public:
	ParamDevice();
	virtual dword Color() { return 0x0f4dddd; };
	virtual int GetDeviceFamily() { return TYPE_PARAM; };

	virtual bool isViewable() { return false; };	///< Parameter Devices do not in of themselves produce a viewable output
	virtual bool isMaskable() { return false; };	///< Parameters cannot be masked

	virtual const char *GetInputName(int slot);
	virtual const char *GetOutputName(int slot);

	virtual bool isOverrideCapable();

};

class ParamGenerator : public ParamDevice {
public:
	ParamGenerator();
	virtual dword Color() override { return 0x0ddefdd; };
};

/// Scalar-based device family. These devices are extremely simple and simply produce or consume scalar (0..1) values.
/// @ingroup devices
class ScalarDevice : public ParamDevice {
public:
	ScalarDevice();

	virtual const char *GetInputName(int slot);
	virtual const char *GetOutputName(int slot);
};

/// Coordinate devices
/// Coordinate devices produce or consume coordinate informatoin. This is currently extremely basic
/// @ingroup devices
class CoordDevice : public ParamDevice {
public:
	CoordDevice();

	virtual const char *GetInputName(int slot);
	virtual const char *GetOutputName(int slot);
};

class CurveDevice : public ParamDevice {
public:
	virtual dword Color() { return 0x0f4f4dd; };
	virtual int GetDeviceFamily() { return TYPE_CURVE; };

	virtual bool isViewable() { return true; };	///< Curves do have a viewable output
	virtual bool isMaskable() { return false; };	///< Curves cannot be masked
};