#pragma once

namespace WorldMachine {

/// A function object class encapsulating a 1D curve.
class Curve1DInterface {
public:
	/// Free this object. It will be deleted from the heap it was allocated from.
	virtual bool Release() = 0;

	/// Clone this curve. Return a new, separately allocated Curve.
	virtual Curve1DInterface *Clone() = 0;

	/// Evaluate the curve at point x and return the result
	virtual double evaluatePoint(double x) = 0;

	/// Return true if the curve is a 0..1 => 0..1 mapping.
	virtual bool isNormalized() const =0;

	/// Return true if the curve is periodic (output tiles perfectly with a repeating feature size of domain)
	virtual bool isExtendedDomain() const = 0;

	/// Retrieve the range (output) of the curve
	virtual Vector2 getRange() const = 0;

	/// Retrieve the domain (input) of the curve
	virtual Vector2 getDomain() const =0;
};

/// Function object representing a space curve, parameterized by t.
class SpaceCurveInterface {
public:
	/// Free this object. It will be deleted from the heap it was allocated from.
	virtual bool Release() = 0;
	/// Clone this curve. Return a new, separately allocated Curve.
	virtual SpaceCurveInterface *Clone() = 0;

	/// Load from disk/store
	virtual bool Load(std::istream &in) = 0;
	/// Save to disk
	virtual void Save(std::ostream &out) = 0;

	/// Evaluate the curve at point t and return the resulting Vector3
	virtual Vector3 evaluatePoint(float t) = 0;

	/// Retrieve the range (output) of the curve in the X axis
	virtual Vector2 getRangeX() = 0;
	virtual Vector2 getRangeY() = 0;
	virtual Vector2 getRangeZ() = 0;

	/// Retrieve the domain (input) of the curve, as x = tmin and y=tmax
	virtual Vector2 getDomain() =0;

	/// Retrieve the total path length of this curve
	virtual double getPathLength() = 0;

};

}
