#pragma once
#include "AreaPacket.h"
#include "Eigen\Dense"
/** Packet carrying Affine Transformation

	@ingroup data
*/
class TransformPacket :
	public WMPacket
{
public:
	virtual PacketPointer Clone() const;
	TransformPacket &operator=(const TransformPacket &other);

	virtual size_t getHeapUsage() { return 0;	};
	static WorldMachine::PacketType getStaticType() { return WorldMachine::Packet::transform; };
	virtual WorldMachine::PacketType getType() const { return getStaticType(); };
	virtual const char *getTypeName() const { return "Transform"; };
	virtual bool queryType(WorldMachine::PacketType const&type) const;	

	virtual bool Load(std::istream &input);
	virtual bool Save(std::ostream &output);

	virtual dword Color() { return 0x0ffff00; };
	virtual void Clear();

	CoordF get() { return location; };
	void set(CoordF &_l) { location = _l; coord_mode = 1; };

	/// Set the transform from a given 3x3 matrix. newmatrix must be an array of 9 floats.
	void setMatrix(Eigen::Matrix3f newmatrix) { matrix = newmatrix; };
	Eigen::Matrix3f getMatrix() const { return matrix; };

	bool hasOrigin() { return coord_mode != 0; };
	void setUseOrigin(bool use_origin) { coord_mode = use_origin ? 1 : 0; };

protected:
	friend class WMPacketFactory;
	TransformPacket();
	virtual ~TransformPacket();
	virtual void resetContents() { };
// LEGACY DATA
	CoordF	location;		// coordinates
	int coord_mode;			// 0 = no location, 1 = set origin

// NEW DATA
	Eigen::Matrix3f matrix;		// 3x3 2D affine transformation matrix

};

/// Safe Packet Cast function 
/// @relates CoordPacket
inline TransformPointer TRANSFORM(PacketPointer const & pack) { 
	if (pack) {
		bool ok = pack->getType() == WorldMachine::Packet::transform;
		if (!ok)
			return NULL;
	}
	return TransformPointer((TransformPacket*) pack.get()); 
};
